<?php

namespace Drupal\testimonials_block\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\Node;
use Drupal\file\Entity\File;

/**
 * Provides the Testimonials Block.
 *
 * @Block(
 *   id="testimonials_block",
 *   admin_label = @Translation("Testimonials Block"),
 * )
 */
class TestimonialsBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'testimonials_block' => [],
    ];
  }

  /**
   * Overrides \Drupal\Core\Block\BlockBase::blockForm().
   *
   * Adds body and description fields to the block configuration form.
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    $items = [];
    if (isset($config['testimonials_data'])) {
      $items = $config['testimonials_data'];
      if($items != '') {
        $items = json_decode($items);
      }
    }

    $form['testimonial_header'] = array(
      '#type' => 'text_format',
      '#title' => t('Testimonial header content'),
      '#description' => t('Testimonial header content'),
      '#default_value' => $config['testimonial_header']??'',
      '#format' => $config['testimonial_header_format']??'basic_html',
    );

    $form['#tree'] = TRUE;

    $form['items_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Testimonials'),
      '#prefix' => '<div id="items-fieldset-wrapper">',
      '#suffix' => '</div>',
    ];


    if (!$form_state->has('num_items')) {
      if(is_array($items)) {
        $count_items = count($items);
      } else {
        $count_items = 0;
      }
      $form_state->set('num_items', $count_items);
    }
    $number_of_items = $form_state->get('num_items');

    for ($i = 0; $i < $number_of_items; $i++) {

      $is_active_row = $form_state->get("row_" . $i );

      if($is_active_row != 'inactive') {

        $j = $i+1;
        $form['items_fieldset']['items'][$i] = [
          '#type' => 'details',
          '#title' => $this->t('Testimonial '. $j),
          '#prefix' => '<div id="items-fieldset-wrapper">',
          '#suffix' => '</div>',
          '#open' => TRUE,
        ];
        $form['items_fieldset']['items'][$i]['author_image'] = [
         '#title' => t('Author image'),
         '#type' => 'managed_file',
         '#upload_location' => 'public://module-images/home-slider-images/',
         '#multiple' => FALSE,
         '#description' => t('Allowed extensions: gif png jpg jpeg.<br>The recommended aspect ratio of an image size is 1:1'),
         '#upload_validators' => [
          'file_validate_is_image' => array(),
          'file_validate_extensions' => array('gif png jpg jpeg'),
          'file_validate_size' => array(25600000)
        ],
        '#theme' => 'image_widget',
        '#preview_image_style' => 'medium',
        '#default_value' => $items[$i]->author_image ? [$items[$i]->author_image[0]] : NULL,
      ];
      $form['items_fieldset']['items'][$i]['quote'] = [
        '#type' => 'textarea',
        '#title' => t('Quote'),
        '#description' => t('Testimonial content'),
        '#default_value' => $items[$i]->quote??'',
      ];
      $form['items_fieldset']['items'][$i]['author'] = [
        '#type' => 'textfield',
        '#title' => t('Author name'),
        '#description' => t('Author name'),
        '#default_value' => $items[$i]->author??'',
      ];
      $form['items_fieldset']['items'][$i]['designation'] = [
        '#type' => 'textfield',
        '#title' => t('Designation'),
        '#description' => t('Author designation'),
        '#default_value' => $items[$i]->designation??'',
      ];
      $form['items_fieldset']['items'][$i]['other_info'] = [
        '#type' => 'textfield',
        '#title' => t('Other info'),
        '#description' => t('Author other extra information data'),
        '#default_value' => $items[$i]->other_info??'',
      ];
      $form['items_fieldset']['items'][$i]['weight'] = [
        '#type' => 'number',
        '#min' => 1,
        '#max' => 120,
        '#title' => t('Testimonial order weight'),
        '#description' => t('The weight field can be used to provide customized sorting of testimonials.'),
        '#default_value' => $items[$i]->weight??$j,
      ];
      $form['items_fieldset']['items'][$i]['remove_single_item'] = [
        '#type' => 'submit',
        '#value' => t('Remove item'),
        '#name' => 'row_'.$i,
        '#submit' => [[$this, 'removeItemCallback']],
        '#ajax' => [
          'callback' => [$this, 'addmoreCallback'],
          'wrapper' => 'items-fieldset-wrapper',
        ],
        '#button_type' => 'danger',
      ];

    }
  }

  $form['items_fieldset']['actions'] = [
    '#type' => 'actions',
  ];

  $form['items_fieldset']['actions']['add_item'] = [
    '#type' => 'submit',
    '#value' => t('Add testimonial'),
    '#submit' => [[$this, 'addOne']],
    '#ajax' => [
        'callback' => [$this, 'addmoreCallback'], //'\Drupal\testimonials\Plugin\Block\ALaUneBlock::addmoreCallback',
        'wrapper' => 'items-fieldset-wrapper',
      ],
      '#button_type' => 'primary',
    ];

    $form['carousel_configuration'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Carousel configuration'),
      '#description' => $this->t('Make adjustments such as how many items will appear according to screen sizes and other configurations.'),
      '#prefix' => '<div id="items-fieldset-wrapper">',
      '#suffix' => '</div>',
    ];

    $form['carousel_configuration']['extra_small'] = [
      '#type' => 'select',
      '#title' => t('Number of items for extra small device.'),
      '#description' => t('Breakpoint from 0 up. Eg. mobile device'),
      '#options' => array(
        '1' => '1'
      ),
      '#default_value' => $config['small']??'1',
    ];

    $form['carousel_configuration']['small'] = [
      '#type' => 'select',
      '#title' => t('Number of items for small device '),
      '#description' => t('Breakpoint from 768 up. Eg. mobile portrait mode'),
      '#options' => array(
        '1' => '1',
        '2' => '2',
      ),
      '#default_value' => $config['small']??'1',
    ];

    $form['carousel_configuration']['medium'] = [
      '#type' => 'select',
      '#title' => t('Number of items for medium device '),
      '#description' => t('Breakpoint from 992 up. Eg. ipad-device'),
      '#options' => array(
        '1' => '1',
        '2' => '2',
        '3' => '3',
      ),
      '#default_value' => $config['medium']??'2',
    ];

    $form['carousel_configuration']['large'] = [
      '#type' => 'select',
      '#title' => t('Number of items for large device '),
      '#description' => t('Breakpoint from 1200 up. Eg. desktop and large screen'),
      '#options' => array(
        '1' => '1',
        '2' => '2',
        '3' => '3',
        '4' => '4',
      ),
      '#default_value' => $config['large']??'3',
    ];

    $form['carousel_configuration']['extra_small_nav'] = [
      '#type' => 'checkbox',
      '#title' => t('Show nav icons for extra small devices'),
      '#description' => t('Breakpoint from 0 up. Eg. mobile device'),
      '#default_value' => $config['extra_small_nav']??false,
    ];

    $form['carousel_configuration']['small_nav'] = [
      '#type' => 'checkbox',
      '#title' => t('Show nav icons for small device '),
      '#description' => t('Breakpoint from 768 up. Eg. mobile portrait mode'),
      '#default_value' => $config['small_nav']??false,
    ];

    $form['carousel_configuration']['medium_nav'] = [
      '#type' => 'checkbox',
      '#title' => t('Show nav icons for medium device '),
      '#description' => t('Breakpoint from 992 up. Eg. ipad-device'),
      '#default_value' => $config['medium_nav']??true,
    ];

    $form['carousel_configuration']['large_nav'] = [
      '#type' => 'checkbox',
      '#title' => t('Show nav icons for large device '),
      '#description' => t('Breakpoint from 1200 up. Eg. desktop and large screen'),
      '#default_value' => $config['large_nav']??true,
    ];

    $form['carousel_configuration']['extra_small_dots'] = [
      '#type' => 'checkbox',
      '#title' => t('Show dots for extra small device.'),
      '#description' => t('Breakpoint from 0 up. Eg. mobile device'),
      '#default_value' => $config['extra_small_dots']??false,
    ];

    $form['carousel_configuration']['small_dots'] = [
      '#type' => 'checkbox',
      '#title' => t('Show dots for small device '),
      '#description' => t('Breakpoint from 768 up. Eg. mobile portrait mode'),
      '#default_value' => $config['small_dots']??false,
    ];

    $form['carousel_configuration']['medium_dots'] = [
      '#type' => 'checkbox',
      '#title' => t('Show dots for medium device '),
      '#description' => t('Breakpoint from 992 up. Eg. ipad-device'),
      '#default_value' => $config['medium_dots']??true,
    ];

    $form['carousel_configuration']['large_dots'] = [
      '#type' => 'checkbox',
      '#title' => t('Show dots for large device '),
      '#description' => t('Breakpoint from 1200 up. Eg. desktop and large screen'),
      '#default_value' => $config['large_dots']??true,
    ];
    $form['carousel_configuration']['hide_box_shadow'] = [
      '#type' => 'checkbox',
      '#title' => t('Hide box shadow'),
      '#description' => t('Hide box shadow for the items'),
      '#default_value' => $config['hide_box_shadow']??false,
    ];

    return $form;
  }


     /**
     * Submit handler for the "remove one" button.
     *
     * Decrements the max counter and causes a form rebuild.
     */
     public function removeItemCallback(array &$form, FormStateInterface $form_state) {
      $button_clicked = $form_state->getTriggeringElement()['#name'];

      $form_state->set($button_clicked, 'inactive');

      $form_state->setRebuild();

    }


  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function addOne(array &$form, FormStateInterface $form_state) {
    $number_of_items = $form_state->get('num_items');
    $add_button = $number_of_items + 1;
    $form_state->set('num_items', $add_button);
    $form_state->setRebuild();
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @return mixed
   */
  public function addmoreCallback(array &$form, FormStateInterface $form_state) {
    // The form passed here is the entire form, not the subform that is
    // passed to non-AJAX callback.
    return $form['settings']['items_fieldset'];
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function removeCallback(array &$form, FormStateInterface $form_state) {
    $number_of_items = $form_state->get('num_items');
    if ($number_of_items > 1) {
      $remove_button = $number_of_items - 1;
      $form_state->set('num_items', $remove_button);
    }
    $form_state->setRebuild();
  }
  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function removeCallbackSingle(array &$form, FormStateInterface $form_state) {
    $number_of_items = $form_state->get('num_items');
    if ($number_of_items > 1) {
      $remove_button = $number_of_items - 1;
      $form_state->set('num_items', $remove_button);
    }
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {

    $testimonials_data_config = $this->configuration['testimonials_data']??'';
    $getTestimonialsOldImageIds = getTestimonialsOldImageIds($testimonials_data_config);
    $current_image_ids = array();
    $items = array();

    $values = $form_state->getValues();
    foreach ($values as $key => $value) {

      if ($key === 'items_fieldset') {
        if (isset($value['items']) && !empty($value['items'])) {
          $items = $value['items'];

          usort($items, function($x, $y) {
            if(is_numeric($x['weight']) && is_numeric($y['weight'])) {
              return $x['weight'] - $y['weight'];
            }
          });

          foreach ($items as $key => $item) {
            if (trim($item['quote']) === '') {
              unset($items[$key]);
            } else {
              if(!is_numeric($item['weight'])) {
                $items[$key]['weight'] = 1;
              }

              if(isset($item['author_image'][0])) {
                $image_id = $item['author_image'][0];
                $file = File::load( $image_id );
                $file->setPermanent();
                $file->save();
                $current_image_ids[] = $image_id;
              }
            }
          }

          $testimonials_data = array_values($items);
          $testimonials_data = json_encode($testimonials_data);

          $this->configuration['testimonials_data'] = $testimonials_data;
        } else {
          $this->configuration['testimonials_data'] = '';
        }

        // remove old images
        $result = array_diff($getTestimonialsOldImageIds, $current_image_ids);
        if(!empty($result)) {
          foreach ($result as $key => $author_image_id) {
            $file = File::load($author_image_id);
            $file->setTemporary();
            $file->save();
          }
        }
      }
    }

    if($form_state->getValue('carousel_configuration')) {
      $carousel_configuration = $form_state->getValue('carousel_configuration');
      $this->configuration['small'] = $carousel_configuration['small'];
      $this->configuration['medium'] = $carousel_configuration['medium'];
      $this->configuration['large'] = $carousel_configuration['large'];

      $this->configuration['extra_small_nav'] = $carousel_configuration['extra_small_nav'];
      $this->configuration['small_nav'] = $carousel_configuration['small_nav'];
      $this->configuration['medium_nav'] = $carousel_configuration['medium_nav'];
      $this->configuration['large_nav'] = $carousel_configuration['large_nav'];

      $this->configuration['extra_small_dots'] = $carousel_configuration['extra_small_dots'];
      $this->configuration['small_dots'] = $carousel_configuration['small_dots'];
      $this->configuration['medium_dots'] = $carousel_configuration['medium_dots'];
      $this->configuration['large_dots'] = $carousel_configuration['large_dots'];
      $this->configuration['hide_box_shadow'] = $carousel_configuration['hide_box_shadow'];
    }

    $this->configuration['testimonial_header'] = $values['testimonial_header']['value'];
    $this->configuration['testimonial_header_format'] = $values['testimonial_header']['format'];

  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $testimonials_data_config = $this->configuration['testimonials_data']??'';
    $small = $this->configuration['small']??'1';
    $medium = $this->configuration['medium']??'2';
    $large = $this->configuration['large']??'2';

    $extra_small_nav = ($this->configuration['extra_small_nav'] == 1)?true:false;
    $small_nav = ($this->configuration['small_nav'] == 1)?true:false;
    $medium_nav = ($this->configuration['medium_nav'] == 1)?true:false;
    $large_nav = ($this->configuration['large_nav'] == 1)?true:false;

    $extra_small_dots = ($this->configuration['extra_small_dots'] == 1)?true:false;
    $small_dots = ($this->configuration['small_dots'] == 1)?true:false;
    $medium_dots = ($this->configuration['medium_dots'] == 1)?true:false;
    $large_dots = ($this->configuration['large_dots'] == 1)?true:false;
    $hide_box_shadow = ($this->configuration['hide_box_shadow'] == 1)?true:false;

    $testimonial_header = $this->configuration['testimonial_header']??'';

    $responsive_settings = array(
      'small' => $small,
      'medium' => $medium,
      'large' => $large,
      'extra_small_nav' => $extra_small_nav,
      'small_nav' => $small_nav,
      'medium_nav' => $medium_nav,
      'large_nav' => $large_nav,
      'extra_small_dots' => $extra_small_dots,
      'small_dots' => $small_dots,
      'medium_dots' => $medium_dots,
      'large_dots' => $large_dots,
      'hide_box_shadow' => $hide_box_shadow
    );

    $testimonials_data = array();
    if($testimonials_data_config != '') {
      $testimonials_data = json_decode($testimonials_data_config);

      foreach ($testimonials_data as $key => $testimonial) {
        $image_url = '';
        $author_image_id = $testimonial->author_image[0]??'';
        if($author_image_id != '') {
          $file = File::load($author_image_id);
          if($file) {
            $image_url = file_create_url($file->getFileUri());
          }
        }
        $testimonials_data[$key]->image_url = $image_url;
      }
    }
    $build = [];
    $build['testimonials'] = [
      '#theme' => 'testimonials_block',
      '#testimonials_data' => $testimonials_data,
      '#responsive_settings' => $responsive_settings,
      '#testimonial_header' => $testimonial_header
    ];
    $build['#attached']['library'][] = 'testimonials_block/testimonials_block';
    return $build;
  }

}